#include "allegro.h"
#include "allegro/internal/aintern.h"
#include "console/console.h"
#include "core/app.h"
#include "core/cfg.h"
#include "core/core.h"
#include "core/dirs.h"
#include "core/modules.h"
#include "core/shutdown.h"
#include "dialogs/about.h"
#include "dialogs/canvasze.h"
#include "dialogs/colsel.h"
#include "dialogs/dfrlen.h"
#include "dialogs/dmapgen.h"
#include "dialogs/dpaledit.h"
#include "dialogs/drawtext.h"
#include "dialogs/effect.h"
#include "dialogs/filesel.h"
#include "dialogs/filmedit.h"
#include "dialogs/gfxsel.h"
#include "dialogs/maskcol.h"
#include "dialogs/maskrepo.h"
#include "dialogs/minipal.h"
#include "dialogs/options.h"
#include "dialogs/playfli.h"
#include "dialogs/quick.h"
#include "dialogs/repo.h"
#include "dialogs/scrsaver.h"
#include "dialogs/tips.h"
#include "dialogs/toolconf.h"
#include "dialogs/vectmap.h"
#include "dialogs/viewspr.h"
#include "effect/colcurve.h"
#include "effect/convmatr.h"
#include "effect/effect.h"
#include "effect/invrtcol.h"
#include "effect/median.h"
#include "effect/replcol.h"
#include "file/ase.h"
#include "file/file.h"
#include "file/gif/format.h"
#include "file/gif/lzw.h"
#include "gfli.h"
#include "intl/intl.h"
#include "intl/msgids.h"
#include "jinete.h"
#include "jinete/alert.h"
#include "jinete/base.h"
#include "jinete/box.h"
#include "jinete/button.h"
#include "jinete/draw.h"
#include "jinete/entry.h"
#include "jinete/hook.h"
#include "jinete/intern.h"
#include "jinete/label.h"
#include "jinete/list.h"
#include "jinete/listbox.h"
#include "jinete/manager.h"
#include "jinete/menu.h"
#include "jinete/message.h"
#include "jinete/rect.h"
#include "jinete/region.h"
#include "jinete/sep.h"
#include "jinete/slider.h"
#include "jinete/system.h"
#include "jinete/textbox.h"
#include "jinete/theme.h"
#include "jinete/view.h"
#include "jinete/widget.h"
#include "jinete/window.h"
#include "lauxlib.h"
#include "libart_lgpl/libart-features.h"
#include "libart_lgpl/libart.h"
#include "lua.h"
#include "lualib.h"
#include "modules/chkmthds.h"
#include "modules/color.h"
#include "modules/editors.h"
#include "modules/gfx.h"
#include "modules/gui.h"
#include "modules/palette.h"
#include "modules/recent.h"
#include "modules/rootmenu.h"
#include "modules/sprites.h"
#include "modules/tools.h"
#include "modules/tools2.h"
#include "png.h"
#include "raster/algo.h"
#include "raster/blend.h"
#include "raster/brush.h"
#include "raster/dirty.h"
#include "raster/frame.h"
#include "raster/gfxobj.h"
#include "raster/image.h"
#include "raster/layer.h"
#include "raster/mask.h"
#include "raster/path.h"
#include "raster/quant.h"
#include "raster/raster.h"
#include "raster/rotate.h"
#include "raster/sprite.h"
#include "raster/stock.h"
#include "raster/undo.h"
#include "script/bindings.h"
#include "script/import.h"
#include "script/script.h"
#include "util/autocrop.h"
#include "util/boundary.h"
#include "util/clipbrd.h"
#include "util/col_file.h"
#include "util/crop.h"
#include "util/filetoks.h"
#include "util/flip.h"
#include "util/frmove.h"
#include "util/hash.h"
#include "util/mapgen.h"
#include "util/misc.h"
#include "util/msk_file.h"
#include "util/pic_file.h"
#include "util/quantize.h"
#include "util/recscr.h"
#include "util/render.h"
#include "util/scrshot.h"
#include "util/setgfx.h"
#include "util/thmbnail.h"
#include "widgets/colbar.h"
#include "widgets/colbut.h"
#include "widgets/colview.h"
#include "widgets/curvedit.h"
#include "widgets/editor.h"
#include "widgets/groupbut.h"
#include "widgets/menu.h"
#include "widgets/paledit.h"
#include "widgets/preview.h"
#include "widgets/statebar.h"
#include "widgets/target.h"
#include "widgets/toolbar.h"
#include "zlib.h"
#include <allegro.h>
#include <allegro/base.h>
#include <allegro/color.h>
#include <allegro/config.h>
#include <allegro/draw.h>
#include <allegro/file.h>
#include <allegro/gfx.h>
#include <allegro/internal/aintern.h>
#include <allegro/keyboard.h>
#include <allegro/system.h>
#include <allegro/unicode.h>
#include <ctype.h>
#include <errno.h>
#ifdef HAVE_LIBJPEG
#include <jpeglib.h>
#endif
#ifdef HAVE_JPGALLEG
#include <jpgalleg.h>
#endif
#include <limits.h>
#include <math.h>
#include <setjmp.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>

#if 0
#include "ft2build.h"
#include FT_FREETYPE_H
#endif
